# -------------------------------------------------------------------------------------------------------------------------------
# SDG_01_Cluster_Stops.py
# Author: Olivier DRAILY, EC, DG REGIO
# Created on: 2020-05-25
#
# Description: 
# The script creates a point feature dataset of clustered stops (bus/tram/metro/rail/...).
# All stops located within 50 m from each other are considered as a single cluster of stops.
# Each cluster is represented by a single point, located at the centre of the clustered stops. 
# The output feature dataset contains all original points which do not have any neighbouring
# points within a radius of 50 meters, in addition to the points representing the clusters.
# -------------------------------------------------------------------------------------------------------------------------------

# Import modules
# -------------------------------------------------------------------------------------------------------------------------------
from __future__ import print_function
import arcpy, sys, os, time
from arcpy import env

# Local variables
# -------------------------------------------------------------------------------------------------------------------------------
stops_to_cluster = r"Sample.gdb\Sample_all_stops_pt"  # Input point feature dataset with stops to cluster  
out_workspace = r"Sample.gdb"   # Output workspace, a geodatabase   # r"Sample.gdb" 
out_dataset_name = "Sample_clustered_stops_pt"  # Output dataset with clustered stops      
cluster_distance = 50   # All stops located within this distance from another stop are considered as a single cluster of stops 

# Environment
# -------------------------------------------------------------------------------------------------------------------------------
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# -------------------------------------------------------------------------------------------------------------------------------
def printMessage(msg):
    print(msg)  # with Python IDLE 3.x or with from __future__ import print_function
    arcpy.AddMessage(msg) # in ArcGIS

# Main Process
# -------------------------------------------------------------------------------------------------------------------------------
# Delete the dataset out_service_areas if it already exists
if arcpy.Exists(out_dataset_name):
    arcpy.Delete_management(out_dataset_name)

# Make a buffer around each stop
printMessage("Make a buffer around each stop")
# To identify all stops located within 50 meters distance from another stop, buffers must be half of cluster_distance wide
buffer_distance = cluster_distance/2 
arcpy.Buffer_analysis(stops_to_cluster, "Buffers", str(buffer_distance) + " Meters")

# Merge contiguous buffers
printMessage("Merge contiguous buffers")
arcpy.Dissolve_management("Buffers", "BuffersDissolved", "", "", "SINGLE_PART")
arcpy.Delete_management("Buffers")

# Assign buffer ID to stops
printMessage("Assign buffer ID to stops")
arcpy.Identity_analysis(stops_to_cluster, "BuffersDissolved", "IdentityFcl", "ALL", "", "NO_RELATIONSHIPS")
arcpy.Delete_management("BuffersDissolved")

# Stops with the same buffer ID are clustered
printMessage("Stops with the same buffer ID are clustered")
arcpy.MeanCenter_stats("IdentityFcl", out_dataset_name, "", "FID_BuffersDissolved")
arcpy.Delete_management("IdentityFcl")

# Deleting unnecessary fields
arcpy.DeleteField_management(out_dataset_name, ["XCoord", "YCoord", "FID_BuffersDissolved"])

printMessage("End: " + time.strftime("%H:%M:%S", time.localtime()))

