# --------------------------------------------------------------------------------------------------------------------------------------
# SDG_02_Make_Service_Areas.py
# Author: Olivier DRAILY, EC, DG REGIO
# Created on: 2020-05-25
#
# Description: 
# The script creates service areas (catchment areas) of 500 meters walking around the clustered public transport stops using a pedestrian network.
# A road network dataset adapted for pedestrians is needed, as well as the following extension for ArcGIS: Network Analyst.
# The road network has a field which provides the number of minutes needed to walk through a network segment (Shape_Length*60/5000,
#  assuming a walking speed of 5km/h).
# --------------------------------------------------------------------------------------------------------------------------------------

# Import modules
# --------------------------------------------------------------------------------------------------------------------------------------
from __future__ import print_function
import arcpy, sys, os, time
from arcpy import env
arcpy.CheckOutExtension("Network")

# Local variables
# --------------------------------------------------------------------------------------------------------------------------------------
clustered_stops = r"Sample.gdb\Sample_clustered_stops_pt"   # Dataset with clustered stops (bus/tram/metro/rail)    
out_workspace = os.path.dirname(clustered_stops)   # Output workspace, a geodatabase #
out_service_areas = os.path.basename(clustered_stops).replace("_clustered_stops_pt", "_service_areas_pl")   # Output dataset with service area polygons   
pedestrian_network = r"D:\DGREGIO\Data\Network_Pedestrian\NW_ES.gdb\ES\Network_ND"    # Pedestrian network    
walking_distance = "500" # Service areas of 500 m walking are created around clustered stops

# Environment
# --------------------------------------------------------------------------------------------------------------------------------------
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# --------------------------------------------------------------------------------------------------------------------------------------
def printMessage(msg):
    print(msg)  # with Python IDLE 3.x or with from __future__ import print_function
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# --------------------------------------------------------------------------------------------------------------------------------------
# Delete the dataset out_service_areas if it already exists
if arcpy.Exists(out_service_areas):
    arcpy.Delete_management(out_service_areas)

# Service areas will be produced by group of maximum 500 stops to preserve the memory. A loop is made for this.
count_clusters = int(arcpy.GetCount_management(clustered_stops)[0])  # Get the number of features in clustered_stops
steps_liste = range(1, count_clusters, 500) # For instance: [1, 501, 1001, 1501, 1746]
printMessage("Production of service areas in " + str(len(steps_liste)) + " steps")
i = 0
for step in steps_liste:
    i = i + 1
    printMessage(" Processing step " + str(i) + " at " + time.strftime("%H:%M:%S", time.localtime()))
    # Make a feature layer with maximum 500 features
    arcpy.MakeFeatureLayer_management(clustered_stops, "clustered_stops_Layer" + str(i), "OBJECTID >= " + str(step) + " AND OBJECTID < " + str(step + 500))

    # Define the service area layer
    Service_Area = "Service_Area" + str(i)
    arcpy.na.MakeServiceAreaLayer(pedestrian_network, Service_Area, "Meters", "TRAVEL_TO", walking_distance, \
    "SIMPLE_POLYS", "NO_MERGE", "DISKS", "NO_LINES", "OVERLAP", "NO_SPLIT", "", "", "ALLOW_UTURNS", "", \
    "TRIM_POLYS", "100 Meters", "NO_LINES_SOURCE_FIELDS")

    # Load the start point locations around which service areas are created
    arcpy.na.AddLocations(Service_Area, "Facilities", "clustered_stops_Layer" + str(i), "Name UID #;CurbApproach # 0;Attr_Minutes # 0;Attr_Meters # 0;\
     Attr_pedestrian_minutes # 0;Breaks_Minutes # #;Breaks_Meters # #;Breaks_pedestrian_minutes # #", "500 Meters", "", \
     "", "MATCH_TO_CLOSEST", "APPEND", "NO_SNAP", "", "EXCLUDE", "")

    # Make the service areas 
    arcpy.na.Solve(Service_Area, "SKIP", "CONTINUE") 

    # Create the output service area feature class
    if not arcpy.Exists(out_service_areas):
        arcpy.management.CopyFeatures(Service_Area + "\\Polygons", out_service_areas)
    else:
        arcpy.management.Append(Service_Area + "\\Polygons", out_service_areas)
    
    # Cleaning
    arcpy.Delete_management("clustered_stops_Layer" + str(i))
    arcpy.Delete_management(Service_Area)

# Delete the text field Name
arcpy.DeleteField_management(out_service_areas, ["Name"])

# Check in Network Analyst licenses
arcpy.CheckInExtension("Network")

printMessage("End: " + time.strftime("%H:%M:%S", time.localtime()))
