# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# SDG_03_Make_core_indicator.py
# Author: Olivier DRAILY, EC, DG REGIO
# Created on: 2020-05-25
#
# Description:
# The script creates a table with the distribution of population within or outside the service areas. 
# The polygon dataset representing the 500 m accessibility areas around the stops (made in step 2) is intersected with the areas containing the population figures.
# From the intersected areas the script produces a table with, for chosen area(s) of interest (e.g. city, region, FUA, urban centre, etc.), the total population,
# the sum of the population living within the service areas and the sum of population living outside the service areas.
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

# Import modules
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
from __future__ import print_function
import arcpy, sys, os, time
from arcpy import env

# Local variables
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
SA_dts = r"Sample.gdb\Sample_service_areas_pl"    # The Service Areas dataset     
input_popl = r"Sample.gdb\Sample_population_source_pl"  # A polygon dataset with population figures   
popl_field = "POPL_2011"   #  The field with population figures in these dataset
zones_dts = r"Sample.gdb\Sample_interest_areas_pl"    # A polygon dataset of the boundaries of the zones for which we want results (administrative units, urban centres, ...)   
zone_id_field = "ZONE_ID"   # The field with the location id's in the dataset of territorial units (identifiers of the territorial units)   
zone_query = "ZONE_ID in ('GEOSTAT11_674', 'GEOSTAT11_687')" # The query to make an areaOfInterest layer by selecting the location id's in the dataset zones_dts   
out_workspace = r"Sample.gdb" # Output out_workspace, a geodatabase   
outputTable = os.path.basename(SA_dts).replace("_service_areas_pl", "_core_indicator_tbl")  #  OutputTable name

# Environment
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
def printMessage(msg):
    print(msg)  # with Python IDLE 3.x or with from __future__ import print_function
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Delete the dataset out_service_areas if it already exists
if arcpy.Exists(outputTable):
    arcpy.Delete_management(outputTable)
    
# Make a dataset areaOfInterest from the dataset of territorial divisions, using the query 'zone_query' => areaOfInterest
printMessage("Make a dataset areaOfInterest")
# The parameter 'field_mapping' is used to discard all fields but zone_id_field in the attribute table of areaOfInterest
fld = arcpy.ListFields(zones_dts, zone_id_field)[0]
field_mapping = zone_id_field + " \"" + zone_id_field + "\" true true false " + str(fld.length) + " " + fld.type + " 0 0 ,First,#," + zones_dts + "," + zone_id_field + ",-1,-1"
arcpy.FeatureClassToFeatureClass_conversion(zones_dts, out_workspace, "areaOfInterest", zone_query, field_mapping)

# Dissolve the Service Areas dataset
printMessage("Dissolve the Service Areas dataset")
arcpy.Dissolve_management(SA_dts, "SA_Dissolved", "FromBreak", "", "SINGLE_PART")

# Combine the Service Areas dataset with areaOfInterest   => comb_sa_aoi
# Identity is used instead of Intersect to get polygons falling outside SA to later fetch their population figures
printMessage("Combine Service Areas with areaOfInterest")
arcpy.Identity_analysis("areaOfInterest", "SA_Dissolved", "comb_sa_aoi", "ALL")
arcpy.Delete_management("SA_Dissolved")

# Intersect comb_sa_aoi with the dataset containing the population => comb_sa_popl_aoi
printMessage("Intersect the result of previous step with the dataset containing the population")
# Option 'RATIO' to distribute population between cut polygons when using the Intersect command
arcpy.MakeFeatureLayer_management(input_popl, "input_popl_Layer", "", "", popl_field + " " + popl_field + " VISIBLE RATIO")
arcpy.Intersect_analysis(["comb_sa_aoi","input_popl_Layer"], "comb_sa_popl_aoi", "ALL", "", "INPUT")
arcpy.DeleteField_management("comb_sa_popl_aoi", ["FID_Sample_population", "FID_comb_sa_aoi", "FID_areaOfInterest"])
arcpy.Delete_management("input_popl_Layer")
arcpy.Delete_management("comb_sa_aoi")
arcpy.Delete_management("areaOfInterest")

# Make the final table with the sum of the population, group by area of interest
printMessage("Make final table")
arcpy.Statistics_analysis("comb_sa_popl_aoi", outputTable, [[popl_field, "SUM"]], zone_id_field)
arcpy.AddField_management(outputTable, "POPL_TOTL", "LONG")
arcpy.CalculateField_management(outputTable, "POPL_TOTL", "int(round(!SUM_" + popl_field + "!))", "PYTHON_9.3")
arcpy.DeleteField_management(outputTable, "SUM_" + popl_field)
arcpy.DeleteField_management(outputTable, "FREQUENCY")

# Calculate the sum of the population located inside Service Areas, group by area of interest
arcpy.MakeFeatureLayer_management("comb_sa_popl_aoi", "comb_sa_popl_aoi_Layer", "FID_SA_Dissolved > 0")
arcpy.Statistics_analysis("comb_sa_popl_aoi_Layer", "PoplInSA", [[popl_field, "SUM"]], zone_id_field)
arcpy.AddField_management("PoplInSA", "POPL_IN_SA", "LONG")
arcpy.CalculateField_management("PoplInSA", "POPL_IN_SA", "int(round(!SUM_" + popl_field + "!))", "PYTHON_9.3")
arcpy.JoinField_management(outputTable, zone_id_field, "PoplInSA", zone_id_field, ["POPL_IN_SA"])
arcpy.Delete_management("comb_sa_popl_aoi_Layer")
arcpy.Delete_management("comb_sa_popl_aoi")
arcpy.Delete_management("PoplInSA")

# Calculate the sum of the population located outside Service Areas, group by area of interest
arcpy.AddField_management(outputTable, "POPL_OUT_SA", "LONG")
arcpy.CalculateField_management(outputTable, "POPL_OUT_SA", '!POPL_TOTL! - !POPL_IN_SA!', "PYTHON_9.3")

printMessage("End: " + time.strftime("%H:%M:%S", time.localtime()))


