# ---------------------------------------------------------------------------
# 02_Cluster_Stops.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-16
#
# Description: 
# The script creates a point feature dataset of clustered stops.
# It should be run for each group of transport modes:
#   - for bus/tram 
#   - for metro/rail 
# All stops located within 50 m from each other are considered as a single cluster of stops.
# Each cluster is represented by a single point, located at the centre of the clustered stops. 
# The output feature dataset contains all original points which do not have any neighbouring
# points within a radius of 50 meters, in addition to the new clustered points.
# If a stop location represents a cluster of nearby stops, the departures of all clustered locations are summed.
# The average number of departures per hour is then calculated for each of the (clustered) stops.
# ---------------------------------------------------------------------------

# Import modules
# ---------------------------------------------------------------------------
import arcpy, sys, os, time
from arcpy import env

print "Start: " + time.strftime("%H:%M:%S", time.localtime())

# Local variables
stops_to_cluster = r"Test.gdb\Test_bus_tram_pt"  # Input point feature dataset with stops to cluster  # Test_bus_tram_pt    #  Test_metro_rail_pt   
field_sum_depart = "SUM_DEP_06H_19H"   # Field with the total number of departures between 6:00 and 20:00 #   "SUM_DEP_06H_19H" 
number_of_hours = 14   # Considered number of hours of the public transport service. For instance 14 (from 6:00 to 20:00)
cluster_distance = 50   # All stops located within this distance from another stop are considered as a single cluster of stops 
out_workspace = r"Test.gdb"   # Output workspace, a geodatabase # r"Test.gdb" 
out_dataset_name = os.path.basename(stops_to_cluster)  + "_Clustered"

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS
# Main Process
# ---------------------------------------------------------------------------
# Make a buffer around each stop
printMessage("Make a buffer around each stop")
buffer_distance = cluster_distance/2 # To identify all stops located within 50 meters distance from another stop, buffers must be the half of 50 meters
arcpy.Buffer_analysis(stops_to_cluster, "Buffers", str(buffer_distance) + " Meters")

# Merge contiguous buffers
printMessage("Merge contiguous buffers")
arcpy.Dissolve_management("Buffers", "BuffersDissolved", "", "", "SINGLE_PART")
arcpy.Delete_management("Buffers")

# Assign buffer ID to stops
printMessage("Assign buffer ID to stops")
arcpy.Identity_analysis(stops_to_cluster, "BuffersDissolved", "IdentityFcl", "ALL", "", "NO_RELATIONSHIPS")
arcpy.Delete_management("BuffersDissolved")

# Stops with the same buffer ID are clustered
printMessage("Stops with the same buffer ID are clustered")
arcpy.MeanCenter_stats("IdentityFcl", out_dataset_name, "", "FID_BuffersDissolved")

# Make a table with the sum of departures by clustered stop
printMessage("Calculate the sum of departures by clustered stop")
arcpy.Statistics_analysis("IdentityFcl", "IdentityFclStat", field_sum_depart + " SUM", "FID_BuffersDissolved")
arcpy.AlterField_management("IdentityFclStat", "SUM_" + field_sum_depart, "SUM_DEP", "SUM_DEP")  # To rename SUM_SUM_DEP by SUM_DEP
arcpy.Delete_management("IdentityFcl")

# Push the field SUM_DEP in the clustered stops
printMessage("Add the sum of departures to the clustered stops")
arcpy.JoinField_management(out_dataset_name, "FID_BuffersDissolved", "IdentityFclStat", "FID_BuffersDissolved", "SUM_DEP")
arcpy.Delete_management("IdentityFclStat")

# Calculate the average number of departures per hour (SUM_DEP / considered number of hours of service)
printMessage("Calculate the average number of departures per hour")
arcpy.AddField_management(out_dataset_name, "AVG_DEP", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management(out_dataset_name, "AVG_DEP", "!SUM_DEP! / " + str(number_of_hours), "PYTHON_9.3", "")

# Deleting unnecessary fields
arcpy.DeleteField_management(out_dataset_name, ["XCoord", "YCoord", "FID_BuffersDissolved"])

print "End: " + time.strftime("%H:%M:%S", time.localtime())

