# ---------------------------------------------------------------------------
# 03_Make_Service_Areas.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-16
#
# Description: 
# The script creates service areas (catchment areas) around the clustered stops using a pedestrian network.
# Service areas of 500 meters walking are created around bus and tram stops, while 1 km of walking distance is used around metro and rail stops.
# Service areas represent thus the spatial distribution of the average number of departures an hour between 6:00 and 20:00 accessible within walking distance around stops (or clusters of stops).
# ---------------------------------------------------------------------------

# Import modules
# ---------------------------------------------------------------------------
import arcpy, sys, os, time
from arcpy import env
arcpy.CheckOutExtension("Network")

print "Start: " + time.strftime("%H:%M:%S", time.localtime())

# Local variables
# ---------------------------------------------------------------------------
clustered_stops = r"Test.gdb\Test_bus_tram_pt_Clustered"   # Dataset with clustered stops (bus/tram or metro/rail)  #  bus_tram     metro_rail
pedestrian_network = r"D:\DGREGIO\Data\Network_Pedestrian\NW_ES.gdb\ES\Network_ND"   # Pedestrian network
walking_distance_bus_tram = "500" # Service areas of 500 m walking are created around bus and tram stops
walking_distance_metro_rail = "1000" # Service areas of 1000 m walking are created around metro and rail stops
out_workspace = os.path.dirname(clustered_stops)   # Output workspace, a geodatabase #
out_service_areas = os.path.basename(clustered_stops).replace("_pt_Clustered", "_SA")   # Output dataset with service area polygons

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# ---------------------------------------------------------------------------
# Select the service areas walking distance according to the transport mode
if "bus_tram" in clustered_stops:
    walking_distance = walking_distance_bus_tram  
elif "metro_rail" in clustered_stops:
    walking_distance = walking_distance_metro_rail  
else:
    printMessage("The input dataset with clustered stops does not have a name with 'bus_tram' or 'metro_rail'.\n Please adapt the name or the script")
    quit

# Delete the dataset out_service_areas if it already exists
if arcpy.Exists(out_service_areas):
    arcpy.Delete_management(out_service_areas)

# Add a field UID in the dataset clustered_stops in preparation for a join
printMessage("Add a field UID in the dataset clustered_stops")
if not arcpy.ListFields(clustered_stops, "UID"):
    arcpy.AddField_management(clustered_stops, "UID", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED")
arcpy.CalculateField_management(clustered_stops, "UID", "!OBJECTID!", "PYTHON_9.3")

# Service areas will be produced by group of maximum 500 stops to preserve the memory. A loop is made for this.
count_clusters = int(arcpy.GetCount_management(clustered_stops)[0])  # Get the number of features in clustered_stops
steps_liste = range(1, count_clusters, 500) # For instance: [1, 501, 1001, 1501, 1746]
printMessage("Production of service areas in " + str(len(steps_liste)) + " steps")
i = 0
for step in steps_liste:
    i = i + 1
    printMessage(" Processing step " + str(i) + " at " + time.strftime("%H:%M:%S", time.localtime()))
    # Make a feature layer with maximum 500 features
    arcpy.MakeFeatureLayer_management(clustered_stops, "clustered_stops_Layer" + str(i), "OBJECTID >= " + str(step) + " AND OBJECTID < " + str(step + 500))

    # Define the service area layer
    printMessage("  Define the service area layer")
    Service_Area = "Service_Area" + str(i)
    arcpy.MakeServiceAreaLayer_na(pedestrian_network, Service_Area, "Meters", "TRAVEL_TO", walking_distance, \
    "SIMPLE_POLYS", "NO_MERGE", "DISKS", "NO_LINES", "OVERLAP", "NO_SPLIT", "", "", "ALLOW_UTURNS", "", \
    "TRIM_POLYS", "100 Meters", "NO_LINES_SOURCE_FIELDS")

    # Load the start point locations around which service areas are created
    printMessage("  Load the start point locations around which service areas are created")
    arcpy.AddLocations_na(Service_Area, "Facilities", "clustered_stops_Layer" + str(i), "Name UID #;CurbApproach # 0;Attr_Minutes # 0;Attr_Meters # 0;\
     Attr_pedestrian_minutes # 0;Breaks_Minutes # #;Breaks_Meters # #;Breaks_pedestrian_minutes # #", "500 Meters", "", \
     "", "MATCH_TO_CLOSEST", "APPEND", "NO_SNAP", "", "EXCLUDE", "")

    # Make the service areas 
    printMessage("  Make the service areas") 
    arcpy.Solve_na(Service_Area, "SKIP", "CONTINUE") 

    # Create the output service area feature class
    printMessage("  Create the output service area feature class")
    if not arcpy.Exists(out_service_areas):
        arcpy.management.CopyFeatures(Service_Area + "\\Polygons", out_service_areas)
    else:
        arcpy.management.Append(Service_Area + "\\Polygons", out_service_areas)
    
    # Cleaning
    arcpy.Delete_management("clustered_stops_Layer" + str(i))
    arcpy.Delete_management(Service_Area)

# Add a field UID to the dataset out_service_areas to make a join with the dataset of clustered_stops
printMessage("Add a field UID to the dataset out_service_areas to make a join with the dataset of clustered_stops")
if not arcpy.ListFields(out_service_areas, "UID"):  
    arcpy.AddField_management(out_service_areas, "UID", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED")
arcpy.CalculateField_management(out_service_areas, "UID", "int(!Name!.split(' :')[0])", "PYTHON_9.3")

# Join the field AVG_DEP of the dataset clustered_stops to the dataset out_service_areas
printMessage("Join the field AVG_DEP of the dataset clustered_stops to the output Service Areas dataset at " + time.strftime("%H:%M:%S", time.localtime()))
arcpy.JoinField_management(out_service_areas, "UID", clustered_stops, "UID", "AVG_DEP")

# Delete the text field Name
arcpy.DeleteField_management(out_service_areas, ["Name"])

# Check in Network Analyst licenses
arcpy.CheckInExtension("Network")

print "End: " + time.strftime("%H:%M:%S", time.localtime())
