# ---------------------------------------------------------------------------
# 04_Reclassify_Service_Areas.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-16
#
# Description:
# This tool converts the service areas (catchment areas) into polygons in which the average number of departures has been reclassified in four 
# classes recorded in a field FREQ_CLASS_BUS_TRAM or FREQ_CLASS_METRO_RAIL :
#   0 : Outside service areas
#   1 : < 4 departures/hour
#   2 : >= 4 and < 10 departures/hour
#   3 : >= 10 departures an hour
# The tool must be run for each group of transport modes:
#   - for bus/tram 
#   - for metro/train
# Service areas produced with the previous script (03) represent the spatial distribution of the average frequencies of departures between 6:00 and 20:00 around stops.
# Within each of the groups of transport modes, service areas tend to partly overlap each other.
# The present script first calculates the geometric union of the service areas but, at intersection, it keeps the polygon with the highest value,
# as we assume that the stop with the most frequent departures is the most probable choice between the stops nearby.
# Frequencies are then reclassified into the four classes.
# ---------------------------------------------------------------------------

# Import modules
# ---------------------------------------------------------------------------
import arcpy, sys, os, time
from arcpy import env

print "Start: " + time.strftime("%H:%M:%S", time.localtime())
sep = os.path.sep

# Local variables
# ---------------------------------------------------------------------------
clustered_stops = r"Test.gdb\Test_bus_tram_pt_Clustered"  #  Clustered stops feature class (ex: FR_bus_tram_PT_Aggregated) #  metro_rail   bus_tram
service_areas = r"Test.gdb\Test_bus_tram_SA"   #  Service areas feature class, for a transport mode (ex: FR_bus_tram_SA)
field_average_dep = "AVG_DEP"
out_workspace = os.path.dirname(clustered_stops)   # Output workspace, a geodatabase #
outputSaReclassified = os.path.basename(service_areas)  + "_Reclass" # The reclassified service areas
outputFreqClasses = os.path.basename(service_areas).replace("_SA", "_Serv_Freq_4classes") # Dataset where the average number of departures is reclassified in 4 classes

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True
arcpy.env.XYTolerance = "0.0001 Meters"  # Set the XYTolerance to 0.0001 Meters (to avoid the crash of Dissolve)

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# ---------------------------------------------------------------------------
# Set the name of the field with the frequency classes, according to the transport mode
if "bus_tram" in clustered_stops:
    field_freq_class = "FREQ_CLASS_BUS_TRAM"
    field_average_dep_renamed = "AVG_DEP_BUS_TRAM"
elif "metro_rail" in clustered_stops:
    field_freq_class = "FREQ_CLASS_METRO_RAIL"
    field_average_dep_renamed = "AVG_DEP_METRO_RAIL"
else:
    printMessage("The input dataset with clustered stops does not have a name with 'bus_tram' or 'metro_rail'.\n Please adapt the name or the script")
    quit

# Make the geometric union of the service areas
printMessage('Make the geometric union of the service areas')
arcpy.Union_analysis(service_areas, "SA_union", "ALL", "", "GAPS")

# At intersection (where Union creates identical polygons), keep the polygon with the highest value.
# For this purpose, add a text field with the combined XY coordinates of the polygon centroid, from fields INSIDE_X and INSIDE_Y
printMessage('Add a field with the combined XY coordinates of the polygon centroid')
arcpy.AddGeometryAttributes_management("SA_union", "CENTROID_INSIDE")  # Add INSIDE_X and INSIDE_Y
arcpy.AddField_management("SA_union", "CENTROID_COORD", "TEXT", "", "", "120")
printMessage('Calculate the field')
arcpy.CalculateField_management("SA_union", "CENTROID_COORD", "str(!INSIDE_X!) + str(!INSIDE_Y!)", "PYTHON_9.3")
arcpy.AddIndex_management ("SA_union", "CENTROID_COORD", "CENTROID_COORD")

# Then aggregate the identical features based on that combined field, keeping the features with the maximum value
printMessage('Aggregate the identical features based on that field')
arcpy.Dissolve_management("SA_union", "SA_max", "CENTROID_COORD", field_average_dep + " MAX", "SINGLE_PART")
arcpy.AlterField_management("SA_max", "MAX_" + field_average_dep, field_average_dep, field_average_dep)  # To rename SUM_SUM_DEP by SUM_DEP
arcpy.Delete_management("SA_union")

# Aggregate contiguous features having the same value 
printMessage('Aggregate contiguous features having the same value')
arcpy.Dissolve_management("SA_max", "SA_aggregated", field_average_dep, "", "SINGLE_PART")  # Multipart features are not allowed
arcpy.Delete_management("SA_max")

# Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border, to eliminate small and sliver polygons
printMessage('Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border')
arcpy.MakeFeatureLayer_management("SA_aggregated", "SA_aggregated_Layer")
arcpy.SelectLayerByAttribute_management("SA_aggregated_Layer", "NEW_SELECTION", "\"Shape_Area\" < 100")
arcpy.Eliminate_management("SA_aggregated_Layer", outputSaReclassified, "LENGTH", "", "")
arcpy.Delete_management("SA_aggregated_Layer")
arcpy.Delete_management("SA_aggregated")

# Add and calculate a field for the frequency classes
printMessage('Add and calculate a field for the frequency classes')
arcpy.AddField_management(outputSaReclassified, field_freq_class, "SHORT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
Code_Block_FREQ_CLASS = """def category(AvgDep):   
    if AvgDep == 0:
        return 0
    if AvgDep > 0 and AvgDep < 4:
        return 1
    if AvgDep >= 4 and AvgDep < 10:
        return 2
    if AvgDep >= 10:
        return 3"""
expression = "category(!" + field_average_dep + "!)"   #  "category(!AVG_DEP!)"
arcpy.CalculateField_management(outputSaReclassified, field_freq_class, expression, "PYTHON_9.3", Code_Block_FREQ_CLASS)

# Rename AVG_DEP by AVG_DEP_BUS_TRAM or AVG_DEP_METRO_RAIL
arcpy.AlterField_management(outputSaReclassified, field_average_dep, field_average_dep_renamed, field_average_dep_renamed)  

# Make the polygon dataset with the four frequency classes
printMessage('Make the polygon dataset which with the four frequency classes')
arcpy.Dissolve_management(outputSaReclassified, outputFreqClasses, field_freq_class, "", "SINGLE_PART", "DISSOLVE_LINES")

print "End: " + time.strftime("%H:%M:%S", time.localtime())


