# ---------------------------------------------------------------------------
# 05_Combine_Transport_Modes_and_Make_Serv_Freq_Typology.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-16
#
# Description: 
# This script intersects the dataset with the 4 frequency classes of the transport mode bus/tram with the one of metro/rail,
# producing a set of areas containing the combination of the 4x4 possible frequency classes, i.e. a matrix of 16 possible classes.
# It finally dissolves the intersected areas, grouping these 16 classes to obtain a final typology with five categories of frequencies:
#   0 = No access: no easily accessible departures (by none of the modes)
#   1 = Low: less than 4 departures an hour for at least one group of modes, but no access to more than 4 departures an hour;
#   2 = Medium: access to between 4 and 10 departures an hour for at least one group of modes, but no access to more than 10 departures an hour;
#   3 = High: access to more than 10 departures for one group of modes, but not for both;
#   4 = Very high: access to more than 10 departures an hour for both groups of modes.
# ---------------------------------------------------------------------------

# Import modules
# --------------------------------------------------------------------------- 
import arcpy, sys, os, time
from arcpy import env

print "Start: " + time.strftime("%H:%M:%S", time.localtime())
sep = os.path.sep

# Local variables
# ---------------------------------------------------------------------------
bus_tram_FreqClass = r"Test.gdb\Test_bus_tram_Serv_Freq_4classes"   # Dataset in which the average number of departures of bus/tram has been reclassified in 4 classes
#bus_tram_FreqClass = '#'   # In the case there is no bus/tram dataset
metro_rail_FreqClass = r"Test.gdb\Test_metro_rail_Serv_Freq_4classes"   # Dataset in which the average number of departures of metro/rail has been reclassified in 4 classes
#metro_rail_FreqClass = '#'   # In the case there is no metro/rail dataset
outputServFreqTypologyFcl = "Test_Serv_Freq_Typology"    # Name of the output dataset with the five categories of service frequency typology (very high, high, medium, low, no access)

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.overwriteOutput = True

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# ---------------------------------------------------------------------------
# Make the geometric union of the two reclassified service areas datasets (bus/tram and metro/rail)
printMessage("Make the geometric union of the two reclassified service areas datasets (bus/tram and metro/rail)")
inFeatures = [bus_tram_FreqClass, metro_rail_FreqClass]   # The list of the input feature classes
if metro_rail_FreqClass == '#' or not arcpy.Exists(metro_rail_FreqClass):
    inFeatures = [bus_tram_FreqClass]
    printMessage("No metro/rail dataset")
elif bus_tram_FreqClass == '#' or not arcpy.Exists(bus_tram_FreqClass):
    inFeatures = [metro_rail_FreqClass]
    printMessage("No bus/tram dataset")
out_workspace = os.path.dirname(inFeatures[0])  # Output workspace, like a geodatabase
arcpy.env.workspace = out_workspace
arcpy.Union_analysis(inFeatures, "SA_Union")
arcpy.MultipartToSinglepart_management("SA_Union", "SA_SinglePart")
arcpy.Delete_management("SA_Union")

# Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border, to eliminate small and sliver polygons
printMessage("Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border")
arcpy.MakeFeatureLayer_management("SA_SinglePart", "SA_SinglePart_Layer")
arcpy.SelectLayerByAttribute_management("SA_SinglePart_Layer", "NEW_SELECTION", "Shape_Area < 100")
arcpy.Eliminate_management("SA_SinglePart_Layer", "SA_Eliminate", "LENGTH")
arcpy.Delete_management("SA_SinglePart_Layer")
arcpy.Delete_management("SA_SinglePart")
 
# Add the field FREQ_CLASS_BUS_TRAM or the field FREQ_CLASS_METRO_RAIL if absent. This is needed for the calculation below
if metro_rail_FreqClass == '#' or not arcpy.Exists(metro_rail_FreqClass):
    arcpy.AddField_management("SA_Eliminate", "FREQ_CLASS_METRO_RAIL", "SHORT") 
    arcpy.CalculateField_management("SA_Eliminate", "FREQ_CLASS_METRO_RAIL", 0, "PYTHON_9.3")
if bus_tram_FreqClass == '#' or not arcpy.Exists(bus_tram_FreqClass):
    arcpy.AddField_management("SA_Eliminate", "FREQ_CLASS_BUS_TRAM", "SHORT") 
    arcpy.CalculateField_management("SA_Eliminate", "FREQ_CLASS_BUS_TRAM", 0, "PYTHON_9.3")

# Add and calculate the final typology (field FREQ_CATEG) with the five categories of frequencies (0 No access, 1 Low, 2 Medium, 3 High, 4 Very high)
# from the combination of the 4x4 possible frequency classes recorded in the fields FREQ_CLASS_BUS_TRAM and FREQ_CLASS_METRO_RAIL :
#   0: Outside service areas
#   1: < 4 departures/hour
#   2: >= 4 and < 10 departures/hour
#   3: >= 10 departures per hour
printMessage("Add and calculate the field with the final typology")
arcpy.AddField_management("SA_Eliminate", "FREQ_CATEG", "SHORT") 
expression = "getTypology(!FREQ_CLASS_BUS_TRAM!,!FREQ_CLASS_METRO_RAIL!)"
codeblock = """def getTypology(FREQ_CLASS_BUS_TRAM, FREQ_CLASS_METRO_RAIL):
    if FREQ_CLASS_BUS_TRAM == 0 and FREQ_CLASS_METRO_RAIL == 0:
        value = 0
    elif (FREQ_CLASS_BUS_TRAM == 1 and  FREQ_CLASS_METRO_RAIL == 1) or (FREQ_CLASS_BUS_TRAM == 1 and FREQ_CLASS_METRO_RAIL == 0) or (FREQ_CLASS_BUS_TRAM == 0 and  FREQ_CLASS_METRO_RAIL == 1):
        value = 1
    elif FREQ_CLASS_BUS_TRAM == 3 and FREQ_CLASS_METRO_RAIL == 3:
        value = 4
    elif FREQ_CLASS_BUS_TRAM == 3 or FREQ_CLASS_METRO_RAIL == 3:
        value = 3
    elif FREQ_CLASS_BUS_TRAM == 2 or FREQ_CLASS_METRO_RAIL == 2:
        value = 2
    return value"""
arcpy.CalculateField_management("SA_Eliminate", "FREQ_CATEG", expression, "PYTHON_9.3", codeblock)

# The contiguous polygons can now be dissolved on the basis of the 5 categories of the typology
printMessage("Dissolve contiguous polygons on the basis of the 5 categories of the typology")
arcpy.Dissolve_management("SA_Eliminate", "SA_Dissolve", "FREQ_CATEG", "", "SINGLE_PART", "DISSOLVE_LINES")
arcpy.Delete_management("SA_Eliminate")

# Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border, to eliminate small and sliver polygons
printMessage("Merge small polygons (Shape_Area < 100) with their neighbouring polygon that has the longest shared border")
arcpy.MakeFeatureLayer_management("SA_Dissolve", "SA_Dissolve_Layer")
arcpy.SelectLayerByAttribute_management("SA_Dissolve_Layer", "NEW_SELECTION", "Shape_Area < 100")
arcpy.Eliminate_management("SA_Dissolve_Layer", outputServFreqTypologyFcl, "LENGTH")
arcpy.Delete_management("SA_Dissolve_Layer")
arcpy.Delete_management("SA_Dissolve")

print "End: " + time.strftime("%H:%M:%S", time.localtime())
