# ---------------------------------------------------------------------------
# 06_Calculate_Popl_Distrib_By_Service_Frequency.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-16
#
# Description: 
# This tool creates a table which makes the relationship between the frequency of departures and the cumulative distribution of the population
# ---------------------------------------------------------------------------

# Import modules
# --------------------------------------------------------------------------- 
import arcpy, sys, os, time
from arcpy import env

print "Start: " + time.strftime("%H:%M:%S", time.localtime())

# Local variables
# ---------------------------------------------------------------------------
bus_tram_SA_reclass = r"Test.gdb\Test_bus_tram_SA_Reclass"   # The reclassified bus/tram service areas
#bus_tram_SA_reclass = '#'   # In case there is no bus/tram dataset
metro_rail_SA_reclass = r"Test.gdb\Test_metro_rail_SA_Reclass"   # The reclassified metro/rail service areas
#metro_rail_SA_reclass == '#'   # In case there is no metro/rail dataset
input_popl = r"Test.gdb\Test_population_pl"    # A polygon dataset with population figures 
popl_field = "POPL_2011" # The field with population figures in these dataset
zones_dts = r"Test.gdb\Test_urban_centres_pl"    # A polygon dataset with the zones for which we want results (administrative units, urban centres, ...)
zone_id_field = "ZONE_ID"   # The field with the location id's in the dataset of territorial units (identifiers of the territorial units)
zone_query = "ZONE_ID in ('GEOSTAT11_674', 'GEOSTAT11_687')" # The query to make an areaOfInterest layer by selecting the location id's in the dataset zones_dts
out_workspace = os.path.dirname(bus_tram_SA_reclass)   # Output out_workspace, a geodatabase
outputTable = "Test_Popl_Distrib_by_Serv_Freq_tbl"    # outputTable name

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# ---------------------------------------------------------------------------
# Make a dataset areaOfInterest using the zone_query from the territorial division dataset => areaOfInterest
printMessage("Make a dataset areaOfInterest")
arcpy.FeatureClassToFeatureClass_conversion(zones_dts, out_workspace, "areaOfInterest", zone_query)

# Select features of bus_tram_SA_reclass intersecting areaOfInterest to limit the spatial extent of the process
if arcpy.Exists(bus_tram_SA_reclass):
    printMessage("Select features of bus_tram_SA_reclass intersecting the layer areaOfInterest") 
    arcpy.MakeFeatureLayer_management(bus_tram_SA_reclass, "bus_tram_SA_reclass_Layer")
    arcpy.SelectLayerByLocation_management ("bus_tram_SA_reclass_Layer", "INTERSECT", "areaOfInterest")
    arcpy.FeatureClassToFeatureClass_conversion("bus_tram_SA_reclass_Layer", out_workspace, "bus_Tram_SA")
    arcpy.Delete_management("bus_tram_SA_reclass_Layer")

# Select features of metro_rail_SA_reclass intersecting areaOfInterest to limit the spatial extent of the process
if arcpy.Exists(metro_rail_SA_reclass):
    printMessage("Select features of metro_rail_SA_reclass intersecting the layer areaOfInterest") 
    arcpy.MakeFeatureLayer_management(metro_rail_SA_reclass, "metro_rail_SA_reclass_Layer")
    arcpy.SelectLayerByLocation_management ("metro_rail_SA_reclass_Layer", "INTERSECT", "areaOfInterest")
    arcpy.FeatureClassToFeatureClass_conversion("metro_rail_SA_reclass_Layer", out_workspace, "metro_Rail_SA")
    arcpy.Delete_management("metro_rail_SA_reclass_Layer")

# Make the geometric union of bus_Tram_SA with metro_Rail_SA => union_SA
# The script foresees the possible absence of one of both transport modes (most likely metro/rail)
printMessage("Make the geometric union of bus_tram_SA_reclass with metro_rail_SA_reclass")
inFeatures = ["bus_Tram_SA", "metro_Rail_SA"]   # The list of the feature classes to union
expression = "!AVG_DEP_BUS_TRAM! + !AVG_DEP_METRO_RAIL!" # The expression used to populate the field AVG_DEP
if not arcpy.Exists("bus_Tram_SA"):
    inFeatures = ["metro_Rail_SA"]
    expression = "!AVG_DEP_METRO_RAIL!"
    printMessage("No bus/tram dataset")
if not arcpy.Exists("metro_Rail_SA"):
    inFeatures = ["bus_Tram_SA"]
    expression = "!AVG_DEP_BUS_TRAM!"
    printMessage("No metro/rail dataset")
arcpy.Union_analysis(inFeatures, "union_SA", "ALL", "", "GAPS")
arcpy.AddField_management("union_SA", "AVG_DEP", "DOUBLE")
arcpy.CalculateField_management("union_SA", "AVG_DEP", expression, "PYTHON_9.3") # Make the sum of departures from both transport groups
for field_name in ["FID_bus_Tram_SA", "FID_metro_Rail_SA", "AVG_DEP_BUS_TRAM", "AVG_DEP_METRO_RAIL", "FREQ_CLASS_BUS_TRAM", "FREQ_CLASS_METRO_RAIL"]:
    if arcpy.ListFields("union_SA", field_name):
        arcpy.DeleteField_management("union_SA", field_name)
for fcl in ["bus_Tram_SA", "metro_Rail_SA"]:
    if arcpy.Exists(fcl):
        arcpy.Delete_management(fcl)

# Merge sliver and small polygons (Shape_Area < 100) with their neighbouring polygon => union_SA_Cleaned
printMessage('Merge sliver and small polygons (Shape_Area < 100) with their neighbouring polygon')
arcpy.MultipartToSinglepart_management("union_SA", "union_SA_SinglePart")
arcpy.Delete_management("union_SA")
arcpy.MakeFeatureLayer_management("union_SA_SinglePart", "union_SA_SinglePart_Layer")
arcpy.SelectLayerByAttribute_management("union_SA_SinglePart_Layer", "NEW_SELECTION", "Shape_Area < 100")
arcpy.Eliminate_management("union_SA_SinglePart_Layer", "union_SA_Cleaned", "LENGTH")
arcpy.DeleteField_management("union_SA_Cleaned", "ORIG_FID")
arcpy.Delete_management("union_SA_SinglePart_Layer")
arcpy.Delete_management("union_SA_SinglePart")

# Combine union_SA_Cleaned with areaOfInterest  => comb_sa_aoi
# Identity is used instead of Intersect to get polygons falling outside SA to later fetch their population figures
printMessage("Combine union_SA_Cleaned with areaOfInterest")
arcpy.Identity_analysis("areaOfInterest", "union_SA_Cleaned", "comb_sa_aoi", "ALL")
arcpy.Delete_management("areaOfInterest")
arcpy.Delete_management("union_SA_Cleaned")

# Intersect comb_sa_aoi with the dataset containing the population => comb_sa_popl
printMessage("Intersect the combined SA dataset with the dataset containing the population")
arcpy.MakeFeatureLayer_management(input_popl, "input_popl_Layer", "", "", popl_field + " " + popl_field + " VISIBLE RATIO")
# Option "RATIO" to distribute population between cut polygons when using the Intersect command
arcpy.Intersect_analysis(["input_popl_Layer","comb_sa_aoi"], "comb_sa_popl", "ALL", "", "INPUT")
arcpy.Delete_management("input_popl_Layer")
arcpy.Delete_management("comb_sa_aoi")

# Add and calculate a field DEPARTURES, the integer of AVG_DEP
arcpy.AddField_management("comb_sa_popl", "DEPARTURES", "LONG")
arcpy.CalculateField_management("comb_sa_popl", "DEPARTURES", "int(!AVG_DEP! + 0.50)", "PYTHON_9.3")

# Put 0 where the field POPL is null
arcpy.MakeFeatureLayer_management("comb_sa_popl", "comb_sa_popl_view")
arcpy.SelectLayerByAttribute_management("comb_sa_popl_view", "NEW_SELECTION", popl_field + " IS NULL")
if int(arcpy.GetCount_management("comb_sa_popl_view").getOutput(0)) > 0:
    arcpy.CalculateField_management("comb_sa_popl_view", popl_field, "0", "PYTHON_9.3", "")
arcpy.Delete_management("comb_sa_popl_view")

# Add and calculate a field PROD_DEP_POPL = product of Departures multiplied by Population
arcpy.AddField_management("comb_sa_popl", "PROD_DEP_POPL", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management("comb_sa_popl", "PROD_DEP_POPL", "!DEPARTURES! * !" + popl_field + "!", "PYTHON_9.3", "")

# Make the final table with the sum of the population and the sum of PROD_DEP_POPL, group by area of interest and by number of departures
printMessage("Make the final table with the sum of the population and the sum of PROD_DEP_POPL, group by area of interest and by number of departures")
arcpy.Statistics_analysis("comb_sa_popl", outputTable, popl_field + " SUM; PROD_DEP_POPL SUM", zone_id_field + "; DEPARTURES")
arcpy.AlterField_management(outputTable, "SUM_" + popl_field, "POPL", "POPL") # Rename SUM_POPL into POPL

# Add a field POPL_TOTL_ZONE to the output table (total population of each zone of interest)
printMessage("Add a field POPL_TOTL_ZONE to the output table (total population of each zone of interest)")
arcpy.Statistics_analysis("comb_sa_popl", "sum_total_popl", popl_field + " SUM", zone_id_field)
arcpy.AlterField_management("sum_total_popl", "SUM_" + popl_field, "POPL_TOTL_ZONE", "POPL_TOTL_ZONE") # Rename SUM_POPL into POPL_TOTL_ZONE
arcpy.JoinField_management(outputTable, zone_id_field, "sum_total_popl", zone_id_field, "POPL_TOTL_ZONE")
arcpy.Delete_management("sum_total_popl")

# Add a field POPL_TOTL_SA to the output table (population located in service areas, group by zone of interest)
printMessage("Add a field POPL_TOTL_SA to the output table (population located in service areas, group by zone of interest)")
arcpy.MakeFeatureLayer_management("comb_sa_popl", "comb_sa_popl_Layer", '"AVG_DEP"> 0')   #   "\"AVG_DEP\" > 0"
arcpy.Statistics_analysis("comb_sa_popl_Layer", "sum_popl_in_SA", popl_field + " SUM", zone_id_field)
arcpy.AlterField_management("sum_popl_in_SA", "SUM_" + popl_field, "POPL_TOTL_SA", "POPL_TOTL_SA") # Rename SUM_POPL into POPL_TOTL_SA
arcpy.JoinField_management(outputTable, zone_id_field, "sum_popl_in_SA", zone_id_field, "POPL_TOTL_SA")
arcpy.Delete_management("sum_popl_in_SA")
arcpy.Delete_management("comb_sa_popl_Layer")
arcpy.Delete_management("comb_sa_popl")

# Add Field Cumulated percentage, DEP_WEIGHT_POPL
printMessage("Add Field Cumulated percentage, DEP_WEIGHT_POPL")
arcpy.AddField_management(outputTable, "CUM_PCT_POPL", "FLOAT", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.AddField_management(outputTable, "DEP_WEIGHT_POPL", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")

# Calculate Field Cumulated percentage, DEP_WEIGHT_POPL
printMessage("Calculate Field Cumulated percentage, DEP_WEIGHT_POPL")
arcpy.MakeTableView_management(outputTable, "outputTable_tview")
for row in set(arcpy.da.SearchCursor(outputTable, zone_id_field)):
    arcpy.SelectLayerByAttribute_management ("outputTable_tview", "NEW_SELECTION", zone_id_field + " = '" + row[0] + "'")
    arcpy.CalculateField_management("outputTable_tview", "CUM_PCT_POPL", "percentIncrease(float( !POPL! ),float( !POPL_TOTL_SA!),float( !POPL_TOTL_ZONE!))", "PYTHON_9.3", "lastValue = 0\\ndef percentIncrease(newValuePOPL,POPL_TOTL_SA,POPL_TOTL_ZONE):\\n global lastValue\\n percentage = lastValue + (newValuePOPL / POPL_TOTL_ZONE) * 100\\n lastValue = percentage \\n return percentage")
    arcpy.CalculateField_management("outputTable_tview", "DEP_WEIGHT_POPL", "!SUM_PROD_DEP_POPL! / !POPL_TOTL_ZONE!", "PYTHON_9.3", "")
    arcpy.SelectLayerByAttribute_management ("outputTable_tview", "CLEAR_SELECTION")
arcpy.Delete_management("outputTable_tview")
arcpy.DeleteField_management(outputTable, "FREQUENCY")

printMessage("End: " + time.strftime("%H:%M:%S", time.localtime()))


