# ---------------------------------------------------------------------------
# 07_Calculate_Popl_Distrib_By_Typology.py
# Author: Olivier DRAILY, EC DG REGIO
# Created on: 2020-04-08
#
# Description: 
# This tool creates a table with the distribution of the population (sum and percentage) by typology of service frequency 
# and within several areas of interest defined in an input SQL query.
# ---------------------------------------------------------------------------

# Import modules
# --------------------------------------------------------------------------- 
import arcpy, sys, os, time
from arcpy import env

print "Start: " + time.strftime("%H:%M:%S", time.localtime())

# Local variables
# ---------------------------------------------------------------------------
popl_distrib_by_freq = r"Test.gdb\Test_Popl_Distrib_by_Serv_Freq_tbl"   # Table with the distribution of the population by service frequency
freq_typology_fcl = r"Test.gdb\Test_Serv_Freq_Typology"   # The dataset with the typology of service frequency 
freq_typology_field = "FREQ_CATEG"   # The field with the typology of service frequency
input_popl = r"Test.gdb\Test_population_pl"    # A polygon dataset with population figures 
popl_field = "POPL_2011"   # The field with population figures in that dataset
zones_dts = r"Test.gdb\Test_urban_centres_pl"    # A polygon dataset with the zones for which we want results (administrative units, urban centres, ...)
zone_query = "ZONE_ID in ('GEOSTAT11_674', 'GEOSTAT11_687')" # The query to make an areaOfInterest layer by selecting the location id's in the dataset zones_dts
zone_id_field = "ZONE_ID"   # The field with the location id's in the dataset of territorial units
#
out_workspace = os.path.dirname(freq_typology_fcl)   # Output workspace, a geodatabase
outputTable = "Test_Popl_Distrib_by_Typology_tbl"    # outputTable name
valueField = "PERC_POPL"

# Environment
# --------------------------------------------------------------------------- 
arcpy.env.workspace = out_workspace
arcpy.env.overwriteOutput = True

# Routine printMessage
# ---------------------------------------------------------------------------
def printMessage(msg):
    print msg  # with Python 2.x
    #print(msg)  # with Python 3.x
    arcpy.AddMessage(msg) # in ArcGIS

# Main process
# ---------------------------------------------------------------------------
# Selection of the polygons from the zone dataset using the query
printMessage("Selection of the polygons from the location dataset")
arcpy.MakeFeatureLayer_management(zones_dts, "Zone_layer", zone_query)

# Overlaying Zone_layer with the dataset 'service frequency typology'
# Identity is used instead of Intersect to get polygons falling outside the SA to later fetch their population
printMessage("Overlaying Zone_layer with the dataset with service frequency typology")
arcpy.Identity_analysis("Zone_layer", freq_typology_fcl, "LocationsFreqTypol", "ALL")
arcpy.Delete_management("Zone_layer")

# Intersecting input_popl with LocationsFreqTypol
printMessage("Intersecting the population dataset with LocationsFreqTypol")
arcpy.MakeFeatureLayer_management(input_popl, "poplFcl_Layer", "", "", popl_field + " " + popl_field + " VISIBLE RATIO")
# Option "RATIO" to distribute population between cut polygons when using the Intersect command
arcpy.Intersect_analysis(["poplFcl_Layer","LocationsFreqTypol"], "PoplFreqTypolFcl", "ALL", "", "INPUT")
arcpy.Delete_management("poplFcl_Layer")
arcpy.Delete_management("LocationsFreqTypol")

# Make a temporary table with the sum of the population, grouped by zone and by typology of service frequency => poplZonesTypoTbl
printMessage("Make a temporary table with the sum of the population, grouped by zone and by typology of service frequency")
arcpy.Statistics_analysis("PoplFreqTypolFcl", "poplZonesTypoTbl", popl_field + " SUM", [zone_id_field, freq_typology_field])
arcpy.AlterField_management("poplZonesTypoTbl", "SUM_" + popl_field, "SUM_POPL", "SUM_POPL")

# Add the total population of each zone to the table poplZonesTypoTbl
printMessage("Add the total population of each zone")
arcpy.Statistics_analysis("PoplFreqTypolFcl", "totalPoplTable", popl_field + " SUM", zone_id_field)
arcpy.AlterField_management("totalPoplTable", "SUM_" + popl_field, "POPL_TOTL_ZONE", "POPL_TOTL_ZONE")
arcpy.Delete_management("PoplFreqTypolFcl")
arcpy.JoinField_management("poplZonesTypoTbl", zone_id_field, "totalPoplTable", zone_id_field, "POPL_TOTL_ZONE")
arcpy.Delete_management("totalPoplTable")

# Calculate the proportion of the population, grouped by zone and by typology of service frequency
printMessage("Calculate the proportion of the population grouped by zone and by typology of service frequency")
arcpy.AddField_management("poplZonesTypoTbl", "PERC_POPL", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
arcpy.CalculateField_management("poplZonesTypoTbl", "PERC_POPL", "100 * !SUM_POPL! / !POPL_TOTL_ZONE!", "PYTHON_9.3", "")

# Make the output table
printMessage("Make the output table")
arcpy.PivotTable_management("poplZonesTypoTbl", "ZONE_ID", freq_typology_field, valueField, outputTable)
arcpy.DeleteField_management(outputTable, "FREQUENCY")
arcpy.Delete_management("poplZonesTypoTbl")

# Add the fields POPL_TOTL_ZONE, POPL_TOTL_SA, SUM_DEP_WEIGHT_POPL
arcpy.AddMessage("Add the fields POPL_TOTL_ZONE, POPL_TOTL_SA, SUM_DEP_WEIGHT_POPL, PropPoplOutSA")
arcpy.Statistics_analysis(popl_distrib_by_freq, "statTable", "DEP_WEIGHT_POPL SUM", "ZONE_ID;POPL_TOTL_ZONE;POPL_TOTL_SA")
fieldNameList = ["POPL_TOTL_ZONE", "POPL_TOTL_SA", "SUM_DEP_WEIGHT_POPL"]
arcpy.JoinField_management(outputTable, "ZONE_ID", "statTable", "ZONE_ID", fieldNameList)
arcpy.Delete_management("statTable")

printMessage("End: " + time.strftime("%H:%M:%S", time.localtime()))



